var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import * as React from 'react';
import { StyleSheet, View } from 'react-native';
import { safe } from '../Annotation';
import { Image } from './Image';
let Background = class Background extends React.Component {
    constructor(props) {
        super(props);
        this.onLoad = (data) => {
            if (this.props.image && this.props.image.onLoad) {
                this.props.image.onLoad(data);
            }
        };
        this.onError = (error) => {
            if (this.props.image.onError) {
                this.props.image.onError(error);
            }
        };
        this.onLayout = (event) => {
            let width = event.nativeEvent.layout.width;
            let height = event.nativeEvent.layout.height;
            if (width > 0 && height > 0) {
                this.fetchImageSize({ width, height });
            }
        };
        this.fetchImageSize = (contract) => {
            const { image } = this.props;
            if (image) {
                Image.getSize(image.source.uri, (width, height) => {
                    this.setImageState(width, height, contract, image.align, image.resizeMode);
                }, (error) => {
                    this.onError(error);
                });
            }
        };
        this.setImageState = (width, height, contract, align, fallback) => {
            if (width === Number.POSITIVE_INFINITY || height === Number.POSITIVE_INFINITY) {
                this.setState({
                    loaded: true
                });
                return;
            }
            if (width > 0 && height > 0) {
                let resize = this.calcResize(align, width, height, contract, fallback);
                this.setState({
                    loaded: true,
                    resizeMode: resize
                });
            }
        };
        this.calcBestFit = (width, height, contract) => {
            if (width > 0 && height > 0 && contract.width > 0 && contract.height > 0) {
                let imageRatio = width / height;
                let contractRatio = contract.width / contract.height;
                if (imageRatio > contractRatio) {
                    return 'contain';
                }
                else {
                    return 'cover';
                }
            }
            return undefined;
        };
        this.calcResize = (align, width, height, contract, fallback) => {
            switch (align) {
                case 'repeat':
                    return 'repeat';
                case 'fill':
                    return 'stretch';
                case 'center':
                    return 'center';
                case 'fit':
                    return this.calcBestFit(width, height, contract);
                default:
            }
            return fallback;
        };
        this.state = {
            loaded: false,
            resizeMode: 'center'
        };
    }
    render() {
        const _a = this.props, { image, style } = _a, otherProps = __rest(_a, ["image", "style"]);
        return (React.createElement(View, Object.assign({ style: [
                {
                    position: 'relative',
                }, style
            ], onLayout: this.onLayout }, otherProps),
            this.renderImage(),
            this.props.children));
    }
    renderImage() {
        if (this.props.image && this.state.loaded) {
            const _a = this.props.image, { align, style } = _a, imageProps = __rest(_a, ["align", "style"]);
            return (React.createElement(Image, Object.assign({}, imageProps, { style: [
                    StyleSheet.absoluteFill,
                    style
                ], onLoad: this.onLoad, onError: this.onError })));
        }
        return null;
    }
};
Background = __decorate([
    safe
], Background);
export { Background };
